using System;

using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.Modules;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.SupportFunctions;

namespace OwnCode {    
    public class VLModule : IVLModule {
        //define wavelength for evaluations
        double wavelength = 532e-9;
        //define refractive index of surrounding medium
        double refractiveIndexSurrounding = 1;
        //define refractive index of substrate
        double refractiveIndexSubstrate = 2;
        //define FOV rectangle (cartesian angles)
        VectorD FOV_CartesianAngles = new VectorD(30, 20);
        
        public void Run() {
            //calculate k0n for surrounding medium and substrate medium
            double k0n_surrounding = MathFunctions.TwoPi * refractiveIndexSurrounding / wavelength;
            double k0n_substrate = MathFunctions.TwoPi * refractiveIndexSubstrate / wavelength;
            
            //define minimum angle for which TIR has to be ensured
            VectorD maximumAngle_TIR = new VectorD(MathFunctions.DegreeToRadian(-0.5 * FOV_CartesianAngles.X), 0);
            //concert angle to unit vector
            Vector3D unitVectorMaximumAngle;
            DirectionConversion.UnitVectorFromCartesianAngles(maximumAngle_TIR.X, maximumAngle_TIR.Y, out unitVectorMaximumAngle);
            //generate maxumAngleTIR into k-vector
            Vector3D kVectorMaximumTIR_In = unitVectorMaximumAngle * k0n_surrounding;
            //calculate maximum period by inverse grating equation
            double maximumPeriod = MathFunctions.TwoPi / (Math.Sqrt((k0n_surrounding * k0n_surrounding) 
                                 - (kVectorMaximumTIR_In.Y * kVectorMaximumTIR_In.Y)) - kVectorMaximumTIR_In.X);
            
            //define minimum angle for which no evancent has to be ensured
            VectorD minimumAngle_Evancescent = new VectorD(MathFunctions.DegreeToRadian(0.5 * FOV_CartesianAngles.X), 
                MathFunctions.DegreeToRadian(0.5 * FOV_CartesianAngles.Y));
            //concert angle to unit vector
            Vector3D unitVectorMinimumAngle;
            DirectionConversion.UnitVectorFromCartesianAngles(minimumAngle_Evancescent.X, minimumAngle_Evancescent.Y, out unitVectorMinimumAngle);
            //generate minimumAngle_Evancescent into k-vector
            Vector3D kVectorMinimumEvanecent_In = unitVectorMinimumAngle * k0n_surrounding;
            //calculate minimum period by inverse grating equation
            double minimumPeriod = MathFunctions.TwoPi / (Math.Sqrt((k0n_substrate * k0n_substrate) - (kVectorMinimumEvanecent_In.Y * kVectorMinimumEvanecent_In.Y)) - kVectorMinimumEvanecent_In.X);
            
            //output of results
            if(minimumPeriod >= maximumPeriod){
                //log maximum period
                Globals.DataDisplay.LogMessage("The specified field of view can not be handled (completely coupled into the substrate) by the current media configuration.");
            }else{
                //log maximum period
                Globals.DataDisplay.LogMessage("Maximum Period = " + PhysicalUnits.FormatPhysicalUnit(maximumPeriod, PhysicalProperty.Length));
                //log minimum period
                Globals.DataDisplay.LogMessage("Minimum Period = " + PhysicalUnits.FormatPhysicalUnit(minimumPeriod, PhysicalProperty.Length));
            }
        }
    }
}